C   Lorenz Model
C
C     by Naoki Sato  August 15, 2001
C
C ########################################

      REAL*8 A,B,F,G
      PARAMETER (A = 0.25D0)
      PARAMETER (B = 4.0D0)
      PARAMETER (F = 8.0D0)
      PARAMETER (G = 1.0D0)

C   Model parameters:
C     A : Parameter A
C     B : Parameter B
C     F : Parameter F
C     G : Parameter G

      REAL*8 T,DT
      PARAMETER (DT=1.0D-3)
      PARAMETER (IX=100000)

C   The size and number of steps:
C     DT : Time step
C     IX : Number of time steps

      REAL*8 X,Y,Z,DX,DY,DZ

C   Prognostic variables:
C     X : Prognostic variable X
C     Y : Prognostic variable Y
C     Z : Prognostic variable Z
C     prefix D : Time derivateve

C ########################################
C   1. INITIALIZATION
C ########################################

      CALL INIT  (T,X,Y,Z)

C   Here, the output file is opened.

      OPEN (10,FILE='output.dat',STATUS='UNKNOWN',
     +      FORM='FORMATTED')

C   The initial values of X, Y and Z are written

      CALL PRINT (T,X,Y,Z)

C #### Do loop 11 starts here ####

C   In roop 11, the time derivateves of X, Y and Z are calculated,
C   and time-integrations are performed for them.

        DO 11 I=1,IX

C ########################################
C   2. TENDENCY
C ########################################

      CALL ODE (A,B,F,G,X,Y,Z,DX,DY,DZ)

C ########################################
C   3. CLOCK
C ########################################

      T = T + DT

C ########################################
C   4. INTEGRATION
C ########################################

      CALL INTEG (DT,DX,X)
      CALL INTEG (DT,DY,Y)
      CALL INTEG (DT,DZ,Z)

C ########################################
C   8. OUTPUT
C ########################################

      IF (MOD(I,10).EQ.0) CALL PRINT (T,X,Y,Z)

   11   CONTINUE

C #### Do loop 11 ends here ####

      CLOSE(10)

      STOP
      END

C ########################################
C   SUBROUTINES
C ########################################

      SUBROUTINE INIT (T,X,Y,Z)
      REAL*8 T,X,Y,Z

C   Initialization of each variable

C   T : Time
C   X : Prognostic variable X
C   Y : Prognostic variable Y
C   Z : Prognostic variable Z

      T = 0.0D0
      X = 1.0D0
      Y = 1.0D0
      Z = 0.0D0
      RETURN
      END

C ========================================

      SUBROUTINE ODE (A,B,F,G,X,Y,Z,DX,DY,DZ)
      REAL*8 A,B,F,G,X,Y,Z,DX,DY,DZ

C   Calculation of tendency of each variable

C   A  : Parameter A
C   B  : Parameter B
C   F  : Parameter F
C   G  : Parameter G
C   X  : Prognostic variable X
C   Y  : Prognostic variable Y
C   Z  : Prognostic variable Z
C   DX : Tendency of X
C   DY : Tendency of Y
C   DZ : Tendency of Z

      DX = - Y**2 - Z**2 - A*X + A*F
      DY = X*Y - B*X*Z - Y + G
      DZ = B*X*Y + X*Z - Z
      RETURN
      END

C ========================================

      SUBROUTINE INTEG (DT,DX,X)
      REAL*8 DT,X,DX

C   Time-integration

C   DT : Time step
C   DX : Time derivative
C   X  : Values at the present step

      X = X + DT * DX
      RETURN
      END

C ========================================

      SUBROUTINE PRINT (T,X,Y,Z)
      REAL*8 T,TT,X,Y,Z

C   T : Time
C   X : Prognostic variable X
C   Y : Prognostic variable Y
C   Z : Prognostic variable Z

   10 FORMAT ((1X,F6.2),3(1X,F7.4))
      WRITE(10,10) T,X,Y,Z
      RETURN
      END
